@@ -220,6 +220,8 @@ id3_open(const char *fname, int mode)
   newtag->tagsz = -1;
   newtag->seekable = 1;
   newtag->version = 3; /* write 2.3 tags by default */
+  newtag->v1.requested = 1; /* also write ID3v1 tags by default */
+  newtag->v1.genre = 255; /* initialize to "unknown" */
 
   return newtag;
 
@@ -474,12 +476,22 @@ static int
 _look_for_v1tag(id3_t tag)
 {
   int sz;
+  unsigned char buf[128];
   if (fseek(tag->fp, -128, SEEK_END) == -1)
     return 0;
-  sz = fread(tag->v1tag, 1, 128, tag->fp);
-  if (sz == 128 && memcmp(tag->v1tag, "TAG", 3) == 0)
-    tag->has_v1tag = 1;
-  return tag->has_v1tag;
+  sz = fread(buf, 1, 128, tag->fp);
+  if (sz == 128 && memcmp(buf, "TAG", 3) == 0) {
+    tag->v1.exists = 1;
+    strncpy(tag->v1.title, buf + 3, 30);
+    strncpy(tag->v1.artist, buf + 33, 30);
+    strncpy(tag->v1.album, buf + 63, 30);
+    strncpy(tag->v1.year, buf + 93, 4);
+    strncpy(tag->v1.comment, buf + 97, 30);
+    /* make sure track field is not part of a v1.0 comment */
+    tag->v1.track = (buf[125] == 0) ? buf[126] : 0;
+    tag->v1.genre = buf[127];
+  }
+  return tag->v1.exists;
 }
 
 static int
@@ -584,7 +596,7 @@ id3_get_size(id3_t tag)
 	return -1;
 
       if (!tag_found) {
-	if (tag->has_v1tag) {
+	if (tag->v1.exists) {
 	  /* check just before id3v1 tag */
 	  tag_found = _look_for_footer(tag, buf, -138);
 	  if (tag_found == -1)
@@ -1385,4 +1397,5 @@ id3_strip(id3_t tag)
 
   tag->frame_hd = tag->frame_tl = NULL;
   tag->nframes = 0;
+  tag->v1.requested = 0;
 }
