@@ -424,6 +424,23 @@ _write_v3_frames(id3_t tag, FILE *out)
 }
 
 
+/* build the 128-byte ID3v1 disk structure */
+static void
+_build_v1_tag(unsigned char *buf, id3_t tag)
+{
+  memset(buf, 0, 128);
+  strcpy(buf, "TAG");
+  strncpy(buf + 3, tag->v1.title, 30);
+  strncpy(buf + 33, tag->v1.artist, 30);
+  strncpy(buf + 63, tag->v1.album, 30);
+  strncpy(buf + 93, tag->v1.year, 4);
+  strncpy(buf + 97, tag->v1.comment, 30);
+  if (buf[125] == 0) /* check if tag is v1.1 */
+    buf[126] = tag->v1.track;
+  buf[127] = tag->v1.genre;
+}
+
+
 /* Figure out size of old tag and new tag, based on the current pad policy */
 static void
 _calculate_tag_sizes(id3_t tag, int *pold_sz, int *pnew_sz)
@@ -554,6 +571,7 @@ id3_write(id3_t tag)
   int old_tagsz, new_tagsz;
   int sz, written, nframes;
   unsigned char copybuf[4096];
+  unsigned char v1buf[128];
   off_t new_offset;
 #if USE_TEMPFILE
   char *tmpfname = NULL, *p;
@@ -581,14 +599,18 @@ id3_write(id3_t tag)
     return -1;
   }
 
-  /* if the version is 2.2, convert to v2.4 */
+  /* if the version is 2.2, convert to v2.3 */
   if (tag->version == 2) {
-    if (id3_set_version(tag, 4) == -1) {
+    if (id3_set_version(tag, ID3_VERSION_2_3) == -1) {
       errno = EINVAL;
       return -1;
     }
   }
 
+  /* if we are to write a v1 tag, build the 128-byte buffer now */
+  if (tag->v1.requested)
+    _build_v1_tag(v1buf, tag);
+
   /*
    * figure old and new sizes, with padding and headers, and set the
    * tag->tagsz field
@@ -607,6 +629,8 @@ id3_write(id3_t tag)
      in place. */
   if (tag->append_req && new_tagsz < old_tagsz)
     write_in_place = 0;
+  if (tag->v1.exists && !tag->v1.requested)
+    write_in_place = 0;
 #endif
 
   if (write_in_place) {
@@ -640,8 +664,8 @@ id3_write(id3_t tag)
     }
 
     /* write ID3v1 tag to temp file, if appending */
-    if (tag->has_v1tag && tag->append_req) {
-      if (fwrite(tag->v1tag, 1, 128, out) < 128)
+    if (tag->v1.requested && tag->append_req) {
+      if (fwrite(v1buf, 1, 128, out) < 128)
 	goto error_free;
     }
 
@@ -659,19 +683,25 @@ id3_write(id3_t tag)
 
     /* If we're not appending, and we have an ID3v1 tag,
      * fseek and write the v1 tag. */
-    if (tag->has_v1tag && !tag->append_req) {
-      if (fseek(tag->fp, -128, SEEK_END) == -1)
+    if (tag->v1.requested && !tag->append_req) {
+      if (fseek(tag->fp, tag->v1.exists ? -128 : 0, SEEK_END) == -1)
 	goto error_free;
-      if (fwrite(tag->v1tag, 1, 128, tag->fp) < 128)
+      if (fwrite(v1buf, 1, 128, tag->fp) < 128)
 	goto error_free;
     }
 
 #if HAVE_FTRUNCATE
     /* truncate the file to the proper length */
-    /* (note that this only happens when we're appending) */
     if (new_tagsz < old_tagsz) {
+      /* (note: this case only happens when we're appending) */
       if (ftruncate(fileno(tag->fp), tag->offset + new_tagsz
-		    + (tag->has_v1tag ? 128 : 0)) == -1)
+		    + (tag->v1.requested ? 128 : 0)) == -1)
+	goto error_free;
+    } else if (tag->v1.exists && !tag->v1.requested) {
+      /* strip the v1 tag */
+      if (fseek(tag->fp, -128, SEEK_END) == -1)
+	goto error_free;
+      if (ftruncate(fileno(tag->fp), (size_t)ftell(tag->fp)) == -1)
 	goto error_free;
     }
 #endif
@@ -753,7 +783,7 @@ id3_write(id3_t tag)
 	}
       }
 
-      if (tag->has_v1tag) {
+      if (tag->v1.exists) {
 	/* we have to put the appended v2 tag before the v1 tag, so back up */
 	if (fseek(tag->fp, -128, SEEK_CUR) == -1)
 	  goto error_free;
@@ -802,10 +832,10 @@ id3_write(id3_t tag)
 
       /* we're not appending, so if we have an ID3v1 tag,
        * fseek and write the v1 tag. */
-      if (tag->has_v1tag) {
-	if (fseek(out, -128, SEEK_END) == -1)
+      if (tag->v1.requested) {
+	if (fseek(out, tag->v1.exists ? -128 : 0, SEEK_END) == -1)
 	  goto error_free;
-	if (fwrite(tag->v1tag, 1, 128, out) < 128)
+	if (fwrite(v1buf, 1, 128, out) < 128)
 	  goto error_free;
       }
 
@@ -813,8 +843,8 @@ id3_write(id3_t tag)
       /*
        * if we *are* appending, copy the v1 tag last
        */
-      if (tag->has_v1tag)
-	if (fwrite(tag->v1tag, 1, 128, out) < 128)
+      if (tag->v1.requested)
+	if (fwrite(v1buf, 1, 128, out) < 128)
 	  goto error_free;
     }
 
